@@ -1,17 +1,16 @@
+from __future__ import absolute_import
+
 import bz2
 import sys
 
-is_py3k = bool(sys.version_info[0] == 3)
 
-if is_py3k:
-    from io import BytesIO as StringIO
-    MAGIC = bytes('BSDIFF40'.encode('latin1'))
+if sys.version_info[0] == 2:
+    from cStringIO import StringIO as BytesIO
 else:
-    from cStringIO import StringIO
-    if sys.version_info[:2] >= (2, 6):
-        MAGIC = bytes('BSDIFF40')
-    else: # 2.5
-        MAGIC = 'BSDIFF40'
+    from io import BytesIO
+
+MAGIC = b'BSDIFF40'
+
 
 import bsdiff4.core as core
 
@@ -20,7 +19,7 @@ def write_patch(fo, len_dst, tcontrol, bdiff, bextra):
     """write a BSDIFF4-format patch to stream 'fo'
     """
     fo.write(MAGIC)
-    faux = StringIO()
+    faux = BytesIO()
     # write control tuples as series of offts
     for c in tcontrol:
         for x in c:
@@ -60,9 +59,8 @@ def read_patch(fi, header_only=False):
 
 
 def read_data(path):
-    fi = open(path, 'rb')
-    data = fi.read()
-    fi.close()
+    with open(path, 'rb') as fi:
+        data = fi.read()
     return data
 
 
@@ -71,7 +69,7 @@ def diff(src_bytes, dst_bytes):
 
     Return a BSDIFF4-format patch (from src_bytes to dst_bytes) as bytes.
     """
-    faux = StringIO()
+    faux = BytesIO()
     write_patch(faux, len(dst_bytes), *core.diff(src_bytes, dst_bytes))
     return faux.getvalue()
 
@@ -84,9 +82,8 @@ def file_diff(src_path, dst_path, patch_path):
     """
     src = read_data(src_path)
     dst = read_data(dst_path)
-    fo = open(patch_path, 'wb')
-    write_patch(fo, len(dst), *core.diff(src, dst))
-    fo.close()
+    with open(patch_path, 'wb') as fo:
+        write_patch(fo, len(dst), *core.diff(src, dst))
 
 
 def patch(src_bytes, patch_bytes):
@@ -94,7 +91,7 @@ def patch(src_bytes, patch_bytes):
 
     Apply the BSDIFF4-format patch_bytes to src_bytes and return the bytes.
     """
-    return core.patch(src_bytes, *read_patch(StringIO(patch_bytes)))
+    return core.patch(src_bytes, *read_patch(BytesIO(patch_bytes)))
 
 
 def file_patch_inplace(path, patch_path):
@@ -102,13 +99,12 @@ def file_patch_inplace(path, patch_path):
 
     Apply the BSDIFF4-format file patch_path to the file 'path' in place.
     """
-    fi = open(patch_path, 'rb')
-    f = open(path, 'r+b')
-    data = f.read()
-    f.seek(0)
-    f.write(core.patch(data, *read_patch(fi)))
-    f.close()
-    fi.close()
+    with open(patch_path, 'rb') as fi:
+        with open(path, 'r+b') as fo:
+            data = fo.read()
+            fo.seek(0)
+            fo.write(core.patch(data, *read_patch(fi)))
+            fo.truncate()
 
 
 def file_patch(src_path, dst_path, patch_path):
@@ -123,8 +119,6 @@ def file_patch(src_path, dst_path, patch_path):
         file_patch_inplace(src_path, patch_path)
         return
 
-    fi = open(patch_path, 'rb')
-    fo = open(dst_path, 'wb')
-    fo.write(core.patch(read_data(src_path), *read_patch(fi)))
-    fo.close()
-    fi.close()
+    with open(patch_path, 'rb') as fi:
+        with open(dst_path, 'wb') as fo:
+            fo.write(core.patch(read_data(src_path), *read_patch(fi)))
